# Package:  udns
# Date:     30th August 2014
# Author:   James Mills, prologic at shortcircuit dot net dot au


"""udns - a micro DNS Server"""


from warnings import filterwarnings


filterwarnings("ignore", "redis-py works best with hiredis.*")


__version__ = "0.0.1"



"""Test Configuration and Fixtures"""


import threading
from time import sleep
from collections import deque


from pytest import fixture

from circuits.core.manager import TIMEOUT
from circuits import handler, BaseComponent, Debugger, Manager


from udns.server import (
    parse_args, parse_hosts, setup_database, setup_logging, Server
)

from .client import Client


class Watcher(BaseComponent):

    def init(self):
        self._lock = threading.Lock()
        self.events = deque()

    @handler(channel="*", priority=999.9)
    def _on_event(self, event, *args, **kwargs):
        with self._lock:
            self.events.append(event)

    def clear(self):
        self.events.clear()

    def wait(self, name, channel=None, timeout=6.0):
        try:
            for i in range(int(timeout / TIMEOUT)):
                if channel is None:
                    with self._lock:
                        for event in self.events:
                            if event.name == name:
                                return True
                else:
                    with self._lock:
                        for event in self.events:
                            if event.name == name and \
                                    channel in event.channels:
                                return True

                sleep(TIMEOUT)
        finally:
            self.events.clear()


class Flag(object):
    status = False


class WaitEvent(object):

    def __init__(self, manager, name, channel=None, timeout=6.0):
        if channel is None:
            channel = getattr(manager, "channel", None)

        self.timeout = timeout
        self.manager = manager

        flag = Flag()

        @handler(name, channel=channel)
        def on_event(self, *args, **kwargs):
            flag.status = True

        self.handler = self.manager.addHandler(on_event)
        self.flag = flag

    def wait(self):
        try:
            for i in range(int(self.timeout / TIMEOUT)):
                if self.flag.status:
                    return True
                sleep(TIMEOUT)
        finally:
            self.manager.removeHandler(self.handler)


def pytest_addoption(parser):
    parser.addoption(
        "--dbhost", action="store", default="localhost",
        help="Redis host to use for testing"
    )


@fixture(scope="session")
def dbhost(request):
    return request.config.getoption("--dbhost")


@fixture(scope="session")
def manager(request):
    manager = Manager()

    def finalizer():
        manager.stop()

    request.addfinalizer(finalizer)

    waiter = WaitEvent(manager, "started")
    manager.start()
    assert waiter.wait()

    if request.config.option.verbose:
        verbose = True
    else:
        verbose = False

    Debugger(events=verbose).register(manager)

    return manager


@fixture(scope="session")
def watcher(request, manager):
    watcher = Watcher().register(manager)

    def finalizer():
        waiter = WaitEvent(manager, "unregistered")
        watcher.unregister()
        waiter.wait()

    request.addfinalizer(finalizer)

    return watcher


@fixture(scope="session")
def server(request, manager, watcher, dbhost):
    argv = ["-b", "0.0.0.0:5300", "--debug", "--dbhost", dbhost]

    args = parse_args(argv)

    logger = setup_logging(args)

    db = setup_database(args, logger)

    hosts = parse_hosts("/etc/hosts")

    server = Server(args, db, hosts, logger)

    server.register(manager)
    watcher.wait("ready")

    def finalizer():
        server.unregister()

    request.addfinalizer(finalizer)

    return server


@fixture(scope="session")
def client(request, manager, watcher, server):
    client = Client("127.0.0.1", 5300)

    client.register(manager)
    watcher.wait("ready")

    def finalizer():
        client.unregister()

    request.addfinalizer(finalizer)

    return client



# Module:   models
# Date:     30th August 2014
# Author:   James Mills, prologic at shortcircuit dot net dot au


"""Data Models"""


from dnslib import ZoneParser
from dnslib import CLASS, QTYPE, RDMAP, RR

from redisco.models import Model
from redisco.models import Attribute, IntegerField, ListField


class Zone(Model):

    name = Attribute(required=True, unique=True)
    ttl = IntegerField(default=0)
    records = ListField("Record")

    def delete(self):
        for record in self.records:
            record.delete()
        super(Zone, self).delete()

    def _add_record(self, rname, rdata, rclass=CLASS.IN, rtype=QTYPE.A, ttl=0):
        record = Record(
            rname=rname, rdata=rdata,
            rclass=rclass, rtype=rtype, ttl=ttl
        )
        record.save()
        self.records.append(record)
        self.save()

    def add_record(self, rname, rdata, **options):
        rclass = options.get("rclass", CLASS.IN)
        rtype = options.get("rtype", QTYPE.A)
        ttl = options.get("ttl", self.ttl)

        rclass = rclass if rclass is not None else CLASS.IN
        rtype = rtype if rtype is not None else QTYPE.A
        ttl = ttl if ttl is not None else self.ttl

        self._add_record(
            self.name
            if rname == "@"
            else "{0:s}.{1:s}".format(rname, self.name),
            rdata,
            rclass=rclass, rtype=rtype, ttl=ttl
        )

    def delete_record(self, rname):
        fullname = (
            self.name
            if rname == "@"
            else
            "{0:s}.{1:s}".format(rname, self.name)
        )

        records = [
            (i, record)
            for i, record in enumerate(self.records)
            if record.rname == fullname
        ]

        if not records:
            return

        i, record = records[0]
        record.delete()

        del self.records[i]
        self.save()

    def load(self, f):
        parser = ZoneParser(f.read())

        for rr in list(parser):
            rname = str(rr.rname)
            rdata = str(rr.rdata)
            rclass = rr.rclass
            rtype = rr.rtype
            ttl = rr.ttl

            self._add_record(rname, rdata, rclass=rclass, rtype=rtype, ttl=ttl)

        self.ttl = parser.ttl
        self.save()

    def export(self):
        out = [
            "$TTL {0:d}".format(self.ttl),
            "$ORIGIN {0:s}".format(self.name.rstrip(".")),
            "",
        ]

        for record in self.records:
            rr = record.rr
            rname = str(rr.rname.stripSuffix(self.name))
            out.append(
                "{0:23s} {1:7d} {2:7s} {3:7s} {4:s}".format(
                    rname, rr.ttl,
                    CLASS.get(rr.rclass),
                    QTYPE.get(rr.rtype),
                    rr.rdata.toZone()
                )
            )

        return "\n".join(out)

    class Meta:
        indicies = ("id", "name",)


class Record(Model):

    rname = Attribute(required=True)
    rdata = Attribute(required=True)

    ttl = IntegerField(default=0)

    rclass = IntegerField(default=CLASS.IN)
    rtype = IntegerField(default=QTYPE.A)

    @property
    def rr(self):
        rdata = RDMAP[QTYPE[self.rtype]](self.rdata)
        return RR(self.rname, self.rtype, self.rclass, self.ttl, rdata)

    class Meta:
        indicies = ("id", "rname", "rclass", "rtype", "ttl", "rdata",)



"""Test Client"""


from dnslib import DNSQuestion, DNSRecord, CLASS, QTYPE

from circuits import Event, Component
from circuits.net.events import write
from circuits.net.sockets import UDPClient


class query(Event):
    """query Event"""


class reply(Event):
    """reply Event"""


class DNS(Component):
    """DNS Protocol Handling"""

    def read(self, peer, data):
        self.fire(reply(DNSRecord.parse(data)))


class Client(Component):

    channel = "client"

    def init(self, server, port):
        self.server = server
        self.port = int(port)

        self.transport = UDPClient(
            ("127.0.0.1", 0), channel=self.channel
        ).register(self)
        self.protocol = DNS(channel=self.channel).register(self)

    def reply(self, a):
        self.a = a

    def query(self, qname, qtype="A", qclass="IN"):
        qtype = QTYPE.reverse[qtype]
        qclass = CLASS.reverse[qclass]

        q = DNSRecord(q=DNSQuestion(qname, qtype, qclass))

        self.fire(write((self.server, self.port), q.pack()))



"""Tests Main"""


import sys
from types import ModuleType
from os.path import abspath, dirname
from subprocess import Popen, STDOUT


def importable(module):
    try:
        m = __import__(module, globals(), locals())
        return type(m) is ModuleType
    except ImportError:
        return False


def main():
    cmd = ["py.test", "-r", "fsxX", "--ignore=tmp"]

    if importable("pytest_cov"):
        cmd.append("--cov=udns")
        cmd.append("--cov-report=html")

    cmd.append(dirname(abspath(__file__)))

    raise SystemExit(Popen(cmd, stdout=sys.stdout, stderr=STDOUT).wait())


if __name__ == "__main__":
    main()



"""Test Core"""


from __future__ import print_function

from operator import attrgetter


from dnslib import A, AAAA, CLASS, QTYPE


from .client import query


def test_basic(client, watcher):
    client.fire(query("google-public-dns-a.google.com"))
    assert watcher.wait("reply", "client")
    assert client.a.get_a().rdata == A("8.8.8.8")


def test_hosts(client, watcher):
    client.fire(query("localhost"))
    assert watcher.wait("reply", "client")
    assert (
        set(map(attrgetter("rclass", "rtype", "rdata.data"), client.a.rr)) ==
        set([
            (CLASS.IN, QTYPE.A, A("127.0.0.1").data),
            (CLASS.IN, QTYPE.AAAA, AAAA("::1").data)
        ])
    )



# Module:   client
# Date:     30th August 2014
# Author:   James Mills, prologic at shortcircuit dot net dot au


"""Client"""


from __future__ import print_function


from os import environ
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType
from code import InteractiveConsole


from dnslib import CLASS, QTYPE

from redisco import connection_setup


from .models import Zone
from . import __version__


def create(args):
    zone = Zone.objects.filter(name=args.zone).first()

    if zone:
        print("Zone {0:s} already exists!".format(args.zone))
        raise SystemExit(1)

    zone = Zone(name=args.zone, ttl=args.ttl)
    zone.save()

    if args.file is not None:
        zone.load(args.file)


def add(args):
    zone = Zone.objects.filter(name=args.zone).first()

    if not zone:
        print("Zone {0:s} not found!".format(args.zone))
        raise SystemExit(1)

    if args.rname and args.rdata:
        zone.add_record(
            args.rname, args.rdata, ttl=args.ttl,
            rclass=getattr(CLASS, args.rclass),
            rtype=getattr(QTYPE, args.rtype)
        )
    else:
        print("Must specify both a name and data!")
        raise SystemExit(1)


def delete(args):
    zone = Zone.objects.filter(name=args.zone).first()

    if not zone:
        print("Zone {0:s} not found!".format(args.zone))
        raise SystemExit(1)

    if args.rname:
        zone.delete_record(args.rname)
    else:
        zone.delete()


def list(args):
    print("\n".join(zone.name for zone in Zone.objects.all()))


def show(args):
    zone = Zone.objects.filter(name=args.zone).first()

    if not zone:
        print("Zone {0:s} not found!".format(args.zone))
        raise SystemExit(1)

    print("\n".join(record.rr.toZone() for record in zone.records))


def export(args):
    zone = Zone.objects.filter(name=args.zone).first()

    if not zone:
        print("Zone {0:s} not found!".format(args.zone))
        raise SystemExit(1)

    print(zone.export())


def dbshell(args):
    vars = {}
    vars.update(globals())
    vars.update(locals())
    console = InteractiveConsole(vars)
    console.interact()


def setup_database(args):
    dbhost = args.dbhost
    dbport = args.dbport

    connection_setup(host=dbhost, port=dbport)


def parse_args():
    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        version=__version__,
    )

    parser.add_argument(
        "--dbhost", action="store",
        default=environ.get("REDIS_PORT_6379_TCP_ADDR", "localhost"),
        dest="dbhost", metavar="HOST", type=str,
        help="set database host to HOST (Redis)"
    )

    parser.add_argument(
        "--dbport", action="store",
        default=int(environ.get("REDIS_PORT_6379_TCP_PORT", "6379")),
        dest="dbport", metavar="PORT", type=int,
        help="set database port to PORT (Redis)"
    )

    # Commands

    subparsers = parser.add_subparsers(
        title="Commands",
        description="Available Commands",
        help="Description"
    )

    # create
    create_parser = subparsers.add_parser(
        "create",
        help="Create a new Zone"
    )
    create_parser.set_defaults(func=create)

    create_parser.add_argument(
        "--ttl", default=None, metavar="TTL", type=int,
        help="Zone default ttl"
    )

    create_parser.add_argument(
        "zone", metavar="ZONE", type=str,
        help="Name of zone to create"
    )

    create_parser.add_argument(
        "file", metavar="FILE", default=None, nargs="?", type=FileType("r"),
        help="Zone file to import"
    )

    # add
    add_parser = subparsers.add_parser(
        "add",
        help="Add a Zone or Record entry"
    )
    add_parser.set_defaults(func=add)

    add_parser.add_argument(
        "--class", default="IN", metavar="RCLASS", type=str,
        dest="rclass", choices=CLASS.reverse.keys(),
        help="Resource class to add"
    )

    add_parser.add_argument(
        "--type", default="A", metavar="RTYPE", type=str,
        dest="rtype", choices=QTYPE.reverse.keys(),
        help="Resource type to add"
    )

    add_parser.add_argument(
        "--ttl", default=None, metavar="TTL", type=int,
        help="Resource ttl to add"
    )

    add_parser.add_argument(
        "zone", metavar="ZONE", type=str,
        help="Zone to add resource to"
    )

    add_parser.add_argument(
        "rname", nargs="?", default=None, metavar="RNAME", type=str,
        help="Resource name to add"
    )

    add_parser.add_argument(
        "rdata", nargs="?", default=None, metavar="RDATA", type=str,
        help="Resoufce data to add"
    )

    # delete
    delete_parser = subparsers.add_parser(
        "delete",
        help="Delete a Zone or Record"
    )
    delete_parser.set_defaults(func=delete)

    delete_parser.add_argument(
        "zone", metavar="ZONE", type=str,
        help="Zone to delete"
    )

    delete_parser.add_argument(
        "rname", nargs="?", default=None, metavar="RNAME", type=str,
        help="Resource name to delete"
    )

    # list
    list_parser = subparsers.add_parser(
        "list",
        help="List Zones"
    )
    list_parser.set_defaults(func=list)

    # show
    show_parser = subparsers.add_parser(
        "show",
        help="Display records of a zone"
    )
    show_parser.set_defaults(func=show)

    show_parser.add_argument(
        "zone", metavar="ZONE", type=str,
        help="Zone to display"
    )

    # export
    export_parser = subparsers.add_parser(
        "export",
        help="Export a Zone"
    )
    export_parser.set_defaults(func=export)

    export_parser.add_argument(
        "zone", metavar="ZONE", type=str,
        help="Zone to export"
    )

    # dbshell
    dbshell_parser = subparsers.add_parser(
        "dbshell",
        help="Interactive DB Shell"
    )
    dbshell_parser.set_defaults(func=dbshell)

    return parser.parse_args()


def main():
    args = parse_args()
    setup_database(args)
    args.func(args)


if __name__ == "__main__":
    main()



"""Server"""


from __future__ import print_function


import logging
from time import sleep
from os import environ, path
from logging import getLogger
from collections import defaultdict
from socket import AF_INET, SOCK_STREAM, socket
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType


from cachetools import LRUCache

from dnslib import DNSQuestion, DNSRecord
from dnslib import A, AAAA, CLASS, QR, QTYPE, RR

from circuits.app import Daemon
from circuits.net.events import write
from circuits.net.sockets import UDPServer
from circuits import Component, Debugger, Event, Timer

from redisco import connection_setup, get_client


from . import __version__
from .models import Record


CNAME = QTYPE.reverse["CNAME"]


class request(Event):
    """request Event"""


class response(Event):
    """response Event"""


class DNS(Component):

    def read(self, peer, data):
        record = DNSRecord.parse(data)
        event = request if record.header.qr == QR.QUERY else response
        return self.fire(event(peer, record))


class Server(Component):

    channel  = "server"

    def init(self, args, db, hosts, logger):
        self.args = args
        self.db = db

        self.hosts = hosts
        self.logger = logger

        self.bind = args.bind
        self.forward = args.forward

        self.peers = {}
        self.requests = {}
        self.cache = LRUCache(maxsize=args.cachesize)

        if args.daemon:
            Daemon(args.pidfile).register(self)

        if args.debug:
            Debugger(events=args.verbose, logger=logger).register(self)

        self.transport = UDPServer(
            self.bind, channel=self.channel
        ).register(self)
        self.protocol = DNS(channel=self.channel).register(self)

    def ready(self, server, bind):
        self.logger.info(
            "DNS Server Ready! Listening on {0:s}:{1:d}".format(*bind)
        )

        # Timer(1, Event.create("ttl"), persist=True, channel=self.channel).register(self)

    def ttl(self):
        for k, rrs in self.cache.items()[:]:
            if any(rr.ttl == 0 for rr in rrs):
                qname, qtype, qclass = k
                self.logger.info(
                    "Expired Entry: {0:s} {1:s} {2:s}".format(
                        CLASS.get(qclass), QTYPE.get(qtype), qname
                    )
                )
                del self.cache[k]
            else:
                for rr in rrs:
                    rr.ttl -= 1

    def request(self, peer, request):
        qname = str(request.q.qname)
        qtype = request.q.qtype
        qclass = request.q.qclass

        key = (qname, qtype, qclass)

        if key in self.cache:
            self.logger.info(
                "Cached Request ({0:s}): {1:s} {2:s} {3:s}".format(
                    "{0:s}:{1:d}".format(*peer),
                    CLASS.get(qclass), QTYPE.get(qtype), qname
                )
            )

            reply = request.reply()
            for rr in self.cache[key]:
                reply.add_answer(rr)
            self.fire(write(peer, reply.pack()))
            return

        if key in self.hosts:
            self.logger.info(
                "Local Hosts Request ({0:s}): {1:s} {2:s} {3:s}".format(
                    "{0:s}:{1:d}".format(*peer),
                    CLASS.get(qclass), QTYPE.get(qtype), qname
                )
            )

            reply = request.reply()
            for rdata in self.hosts[key]:
                rr = RR(
                    qname,
                    rclass=CLASS.IN,
                    rtype=QTYPE.AAAA if ":" in rdata else QTYPE.A,
                    rdata=AAAA(rdata) if ":" in rdata else A(rdata)
                )
                reply.add_answer(rr)

            self.cache[key] = rr

            self.fire(write(peer, reply.pack()))

            return

        records = Record.objects.filter(rname=qname)

        if not records:
            self.logger.info(
                "Request ({0:s}): {1:s} {2:s} {3:s} -> {4:s}:{5:d}".format(
                    "{0:s}:{1:d}".format(*peer),
                    CLASS.get(qclass), QTYPE.get(qtype), qname,
                    self.forward, 53
                )
            )

            lookup = DNSRecord(q=DNSQuestion(qname, qtype, qclass))
            id = lookup.header.id
            self.peers[id] = peer
            self.requests[id] = request

            self.fire(write((self.forward, 53), lookup.pack()))

            return

        self.logger.info(
            "Authoritative Request ({0:s}): {1:s} {2:s} {3:s}".format(
                "{0:s}:{1:d}".format(*peer),
                CLASS.get(qclass), QTYPE.get(qtype), qname
            )
        )

        rr = []
        reply = request.reply()

        if len(records) == 1 and records[0].rtype == CNAME:
            rr.append(records[0].rr)
            records = Record.objects.filter(rname=records[0].rdata)

        for record in records:
            rr.append(record.rr)

        reply.add_answer(*rr)

        self.cache[key] = rr

        self.fire(write(peer, reply.pack()))

    def response(self, peer, response):
        id = response.header.id

        qname = str(response.q.qname)
        qtype = response.q.qtype
        qclass = response.q.qclass

        if id not in self.peers:
            self.logger.info(
                "Unknown Response ({0:s}): {1:s} {2:s} {3:s}".format(
                    "{0:s}:{1:d}".format(*peer),
                    CLASS.get(qclass), QTYPE.get(qtype), qname
                )
            )

            return

        peer = self.peers[id]
        request = self.requests[id]

        key = (str(request.q.qname), request.q.qtype, request.q.qclass)

        reply = request.reply()

        reply.add_answer(*response.rr)

        self.cache[key] = reply.rr

        self.fire(write(peer, reply.pack()))

        del self.peers[id]
        del self.requests[id]


def waitfor(host, port, timeout=10):
    sock = socket(AF_INET, SOCK_STREAM)

    while not sock.connect_ex((host, port)) == 0 and timeout:
        timeout -= 1
        sleep(1)

    if timeout <= 0:
        print("Timed out waiting for {0:s}:{1:d}".format(host, port))
        raise SystemExit(1)


def setup_database(args, logger):
    dbhost = args.dbhost
    dbport = args.dbport

    logger.debug(
        "Waiting for Redis Service on {0:s}:{1:d} ...".format(dbhost, dbport)
    )

    waitfor(dbhost, dbport)

    logger.debug(
        "Connecting to Redis on {0:s}:{1:d} ...".format(dbhost, dbport)
    )

    connection_setup(host=dbhost, port=dbport)

    logger.debug("Success!")

    db = get_client()

    return db


def setup_logging(args):
    logging.basicConfig(
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",

        level=logging.DEBUG if args.debug else logging.INFO,
        stream=args.logfile
    )

    return getLogger(__name__)


def parse_args(args=None):
    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        version=__version__,
    )

    add = parser.add_argument

    add(
        "--debug", action="store_true", default=False,
        dest="debug",
        help="enable debugging mode"
    )

    add(
        "--verbose", action="store_true", default=False,
        dest="verbose",
        help="enable verbose logging"
    )

    add(
        "--logfile", action="store", default="/dev/stdout",
        dest="logfile", metavar="FILE", type=FileType("w"),
        help="write logs to FILE"
    )

    add(
        "--pidfile", action="store", default="udns.pid",
        dest="pidfile", metavar="FILE", type=str,
        help="write process id to FILE"
    )

    add(
        "--dbhost", action="store",
        default=environ.get("REDIS_PORT_6379_TCP_ADDR", "localhost"),
        dest="dbhost", metavar="HOST", type=str,
        help="set database host to HOST (Redis)"
    )

    add(
        "--dbport", action="store",
        default=int(environ.get("REDIS_PORT_6379_TCP_PORT", "6379")),
        dest="dbport", metavar="PORT", type=int,
        help="set database port to PORT (Redis)"
    )

    add(
        "--cachesize", action="store",
        default=int(environ.get("CACHESIZEE", "1024")),
        dest="cachesize", metavar="SIZEe", type=int,
        help="set cache size to SIZE"
    )

    add(
        "-b", "--bind",
        action="store", type=str,
        default="0.0.0.0:53", dest="bind",
        help="Bind to address:[port]"
    )

    add(
        "-d", "--daemon", action="store_true", default=False,
        dest="daemon",
        help="run as a background process"
    )

    add(
        "-f", "--forward",
        action="store", type=str,
        default="8.8.8.8", dest="forward",
        help="DNS server to forward to"
    )

    return parser.parse_args(args)


def parse_hosts(filename):
    hosts = defaultdict(list)

    if path.exists(filename):
        with open(filename, "r") as f:
            for line in f:
                line = line.strip()
                if not line or line.startswith("#"):
                    continue
                tokens = line.split()
                rdata, labels = tokens[0], tokens[1:]
                for label in labels:
                    if not label.endswith("."):
                        label = "{0:s}.".format(label)
                    key = (label, QTYPE.A, CLASS.IN)
                    hosts[key].append(rdata)

    return hosts


def main():
    args = parse_args()

    logger = setup_logging(args)

    db = setup_database(args, logger)

    hosts = parse_hosts("/etc/hosts")

    Server(args, db, hosts, logger).run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


from glob import glob


from setuptools import setup, find_packages


setup(
    name="udns",
    version="0.0.2",
    description="micro dns server",
    long_description="{0:s}\n\n{1:s}".format(
        open("README.rst").read(), open("CHANGES.rst").read()
    ),
    author="James Mills",
    author_email="James Mills, prologic at shortcircuit dot net dot au",
    url="https://github.com/prologic/udns/",
    download_url="https://github.org/com/udns/releases/",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: No Input/Output (Daemon)",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Communications :: Chat :: Internet Relay Chat",
    ],
    license="MIT",
    keywords="micro dns server udns",
    platforms="POSIX",
    packages=find_packages("."),
    include_package_data=True,
    scripts=glob("bin/*"),
    dependency_links=[
        "https://github.com/kiddouk/redisco/archive/master.zip#egg=redisco-0.2.4"  # noqa
    ],
    install_requires=(
        "circuits==3.0",
        "dnslib==0.9.3",
        "redisco==0.2.4",
        "cachetools==0.4.0",
    ),
    entry_points={
        "console_scripts": [
            "udnsd=udns.server:main",
            "udnsc=udns.client:main",
        ]
    },
    test_suite="tests.main.main",
    zip_safe=False
)
